/*
 * Copyright 2011 maru project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.maru.dog.bind;

import static org.maru.common.util.ConditionUtil.isNotNull;

import java.util.Map;
import java.util.Map.Entry;

import org.maru.common.Key;
import org.maru.common.KeyGen;
import org.maru.dog.core.AbstractMarkedPointBuilder;
import org.maru.dog.core.Definition;
import org.maru.dog.core.MarkedPoint;

public class ActualFieldNameBinderBuilder<T, K> extends AbstractBinderBuilder<T, K> {

    @SuppressWarnings("unchecked")
    public ActualFieldNameBinderBuilder(T target, K... inputs) {
        super(new InstanceBinderImpl<T, K>(KeyGen.getKey(target.getClass()), (Class<T>) target.getClass()),
                target, inputs);
    }

    @SuppressWarnings("unused")
    @Override
    protected void validateAndCreateInstanceBinder(
            Definition<T> targetDefinition,
            Map<Key<?>, Definition<K>> inputDefinitions) {

        Key<?> targetTypeKey = KeyGen.getKey(targetDefinition.getType());
        for (Entry<Key<?>, MarkedPoint<T>> entry : ((BindingDefinition<T>)targetDefinition).getMarkedPoints().entrySet()) {
            Key<?> targetBoundKey = entry.getKey();
            TargetMarkedPoint<T> targetMarkedPoint = (TargetMarkedPoint<T>)entry.getValue();
            Class<?> inputClass = null;

            inputClass = targetMarkedPoint.getInputClass();

            BindingContext<T, K> context = createBindingContext(targetMarkedPoint, targetBoundKey, inputDefinitions, inputClass, null);
            if (isNotNull(context)) {
                MemberBinder<T, K> memberBinder = createMemberBinder(context);
                instanceBinder.addMemberBinder(memberBinder);
            }

        }

    }

    @SuppressWarnings("unchecked")
    @Override
    protected Definition<K> createAndStoreInputDefinition(Key<?> key,
            Class<K> inputClass) {
        if (!BindingDefinitionCache.containsFieldNameInputDefiniton(key)) {
            AbstractMarkedPointBuilder<K> builder = new InputActualFieldNamePointBuilder<K>(inputClass);
            builder.build();

            BindingDefinitionCache.applyFieldNameInputDefinition(key, builder.getDefinition());
            return builder.getDefinition();
        } else {
            Definition<K> definition = (Definition<K>) BindingDefinitionCache.getFieldNameInputDefinition(key);
            return definition;
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    protected Definition<T> createAndStoreTargetDefinition(Key<?> key,
            Class<T> targetType) {
        Definition<T> definition = null;
        if (!BindingDefinitionCache.containsFieldNameTargetDefinition(key)) {
            AbstractMarkedPointBuilder<T> builder = new TargetActualFieldNamePointBuilder<T>(targetType);
            builder.build();
            definition = builder.getDefinition();

            BindingDefinitionCache.applyFieldNameTargetDefinition(key, definition);
        } else {
            definition =  (Definition<T>) BindingDefinitionCache.getFieldNameTargetDefinition(key);
        }
        return definition;
    }

}
